package PNI::GUI::Canvas_item::Text;

use Mouse;
use PNI::GUI;

our $VERSION = $PNI::GUI::VERSION;

extends 'PNI::GUI::Canvas_item';

# usa il createWindow, prova a mettere il text di Tk
# se funziona, proprio un' intera finestra

1;
